/****************************************************************************
** $Id: rs_actiondefault.cpp,v 1.7 2003/09/16 00:02:18 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "rs_actiondefault.h"

#include "rs.h"
#include "rs_modification.h"
#include "rs_snapper.h"
#include "rs_selection.h"



/**
 * Constructor.
 */
RS_ActionDefault::RS_ActionDefault(RS_EntityContainer& container,
                                   RS_GraphicView& graphicView)
        : RS_PreviewActionInterface("Default",
                            container, graphicView) {}



RS_ActionDefault::~RS_ActionDefault() {
    //std::cout << "~RS_ActionDefault\n";
    //finish();
    //std::cout << "~RS_ActionDefault: OK\n";
}



void RS_ActionDefault::init(int status) {
    RS_PreviewActionInterface::init(status);
    v1 = v2 = RS_Vector(false);
    snapMode = RS2::SnapFree;
    snapRes = RS2::RestrictNothing;
}



void RS_ActionDefault::trigger() {
    RS_PreviewActionInterface::trigger();

}



void RS_ActionDefault::mouseMoveEvent(RS_MouseEvent* e) {

    RS_Vector mouse = graphicView->toGraph(RS_Vector(e->x(), e->y()));
    RS_Vector relMouse = mouse - graphicView->getRelativeZero();

    RS_DIALOGFACTORY->updateCoordinateWidget(mouse, relMouse);

    switch (getStatus()) {
    case Dragging:
        //v2 = graphicView->toGraph(e->x(), e->y());
		v2 = mouse;

        if (graphicView->toGuiDX(v1.distanceTo(v2))>10) {
            setStatus(SetCorner2);
        }
        break;

    case SetCorner2:
        if (v1.valid) {
            //v2 = snapPoint(e);
			v2 = mouse;
			
            //deleteSnapper();
            deletePreview();
            clearPreview();

            preview->addEntity(new RS_Line(preview,
                                           RS_LineData(RS_Vector(v1.x, v1.y),
                                                       RS_Vector(v2.x, v1.y))));
            preview->addEntity(new RS_Line(preview,
                                           RS_LineData(RS_Vector(v2.x, v1.y),
                                                       RS_Vector(v2.x, v2.y))));
            preview->addEntity(new RS_Line(preview,
                                           RS_LineData(RS_Vector(v2.x, v2.y),
                                                       RS_Vector(v1.x, v2.y))));
            preview->addEntity(new RS_Line(preview,
                                           RS_LineData(RS_Vector(v1.x, v2.y),
                                                       RS_Vector(v1.x, v1.y))));
            drawPreview();
        }

    default:
        break;
    }
}



void RS_ActionDefault::mousePressEvent(RS_MouseEvent* e) {
    if (RS2::qw(e->button())==RS2::LeftButton) {
        switch (getStatus()) {
        case Neutral:
            v1 = graphicView->toGraph(e->x(), e->y());
            setStatus(Dragging);
            break;

        default:
            break;
        }
    }
}



void RS_ActionDefault::mouseReleaseEvent(RS_MouseEvent* e) {
    RS_DEBUG->print("RS_ActionDefault::mouseReleaseEvent()");

    if (RS2::qw(e->button())==RS2::LeftButton) {
        switch (getStatus()) {
        case Dragging: {
                // select single entity:
                RS_Entity* en = catchEntity(e);

                if (en!=NULL) {
                    deleteSnapper();
                    deletePreview();
                    clearPreview();

                    RS_Selection s(*container, graphicView);
                    s.selectSingle(en);

                    RS_DIALOGFACTORY->updateSelectionWidget(
                        container->countSelected());

                    //deleteSnapper();
                    e->accept();
                    setStatus(Neutral);
                } else {
                    setStatus(SetCorner2);
                }
            }
            break;

        case SetCorner2:
            //v2 = snapPoint(e);
            v2 = graphicView->toGraph(e->x(), e->y());

            // select window:
            //if (graphicView->toGuiDX(v1.distanceTo(v2))>20) {
            deleteSnapper();
            deletePreview();
            clearPreview();

            bool cross = (v2.y>v1.y);
            RS_Selection s(*container, graphicView);
            s.selectWindow(v1, v2, true, cross);

            RS_DIALOGFACTORY->updateSelectionWidget(
                container->countSelected());

            setStatus(Neutral);
            e->accept();
            //}

        }
    }
    else if (RS2::qw(e->button())==RS2::RightButton) {
        if (getStatus()==SetCorner2) {
            deletePreview();
            clearPreview();
            deleteSnapper();
            setStatus(Neutral);
            //std::cout << "accept\n";
            e->accept();
        } else {
            //std::cout << "ignore\n";
            //e->ignore();
            deleteSnapper();
            RS_DIALOGFACTORY->requestPreviousMenu();
            e->accept();
        }
        //init(getStatus()-1);
    }
}



void RS_ActionDefault::commandEvent(RS_CommandEvent* e) {
    RS_String c = e->getCommand().lower();

    // if the current action can't deal with the command,
    //   it might be intended to launch a new command
    //if (!e.isAccepted()) {
    // command for new action:
    //RS2::ActionType type = RS_COMMANDS->cmdToAction(c);
    //if (type!=RS2::ActionNone) {
    //graphicView->setCurrentAction(type);
    //return true;
    //}
    //}
}



RS_StringList RS_ActionDefault::getAvailableCommands() {
    RS_StringList cmd;

    //cmd += "line";
    //cmd += "rectangle";

    return cmd;
}


void RS_ActionDefault::updateMouseButtonHints() {
    switch (getStatus()) {
    case Neutral:
        RS_DIALOGFACTORY->updateMouseWidget("", "");
        break;
    case SetCorner2:
        RS_DIALOGFACTORY->updateMouseWidget(tr("Choose second edge"),
                                            tr("Back"));
        break;
    default:
        RS_DIALOGFACTORY->updateMouseWidget("", "");
        break;
    }
}



void RS_ActionDefault::updateMouseCursor() {
    //graphicView->setMouseCursor(RS2::SelectCursor);
}



void RS_ActionDefault::updateToolBar() {
    //RS_DIALOGFACTORY->requestToolBar(RS2::ToolBarMain);
}

// EOF
